%{
/**
 | Lexical analyzer for desktop calculator
 | From the bison manual example, modified by MLO
 | $Id: dtc.lex,v 1.4 2000/04/06 06:08:08 loreti Exp $
**/

#include "dtcdef.h"
#include "dtc.h"

extern symRec *root;

%}

SGN [+-]
DGT [0-9]
LTR [a-zA-Z]
HEX [0-9a-fA-F]
OCT [0-7]
SYM [a-zA-Z0-9_]
EXP [eE]{SGN}?{DGT}+

%%

<<EOF>> { return 0; }

[\t ] { }

{LTR}{SYM}* {
  symRec *p;
  int newNode;

  root = insertNode(root, yytext, &newNode, &p);
  if (newNode) {
    p->type      = SYMBOL;
    p->value.var = 0.0;
  }
  yylval.tptr = p;
  return p->type;
}

"0" {
  yylval.val = 0;
  return NUMBER;
}

(0[xX]{HEX}+)|(0{OCT}+)|([1-9]{DGT}*) {
  yylval.val = strtoul(yytext, 0, 0);
  return NUMBER;
}

({DGT}+{EXP})|(({DGT}+\.)|({DGT}*\.{DGT}+)){EXP}? {
  yylval.val = atof(yytext);
  return NUMBER;
}

"\n"|";"|"+"|"-"|"*"|"/"|"%"|"^"|"="|"("|")" {
  return yytext[0];
}

"?" {
  puts("- Statement:\t\t{ expression | variable=expression | variable }");
  puts("- Input syntax:\t\tstatement [ ; statement [ ; ... ]]");
  puts("- Known functions:\tsin, cos, tan, asin, acos, atan, ln, log, exp"
       ", sqrt,");
  puts("\t\t\ttrunc, frac, round");
  puts("- Known constants:\tpi, e, dtor, rtod");
  puts("- Floating input:\tdecimal (1.2) or scientific (3.4E5) notation");
  puts("- Integer input:\tdecimal (67),  hex (0xA) or octal (012) notation");
  puts("- Special functions:\tlast, hex, oct, stab;  ? for help; ^D to exit");
}

. { int c;
 puts("  Error: syntax error"); 
 while ((c=input()) != EOF) {
   if (c == ';'   ||   c == '\n') return c;
 }
 return 0;
}

%%
